/*
 * gsm_dal.c
 *
 *  Created on: Nov 26, 2018
 *      Author: Vladimir Sustek
 *
 *
 */
/**
  @verbatim
  ==============================================================================
  How to use driver:

  1) Do all the steps from the gsm_driver.c header (e.g. gsm_device_t instance)
  2) Create FSM with functions:
                               gsm_idle_state() - sends empty AT command
                               gsm_SMS_tbread() - open new received SMS
                               gsm_SMS_publish() - cleans SMS and publish it
                               gsm_SMS_del() - deletes previously read SMS
                               gsm_mod_dialed() - increments dialing for reset
                               gsm_mod_hangup() - clears counter for reset
                               gsm_MMS_send() - send MMS
  3) After the FSM start the timer and process the response using gsm_rec_respnd2()
  4) Implement gsm_missed_response() for GSM module interfacing problem solving
  5) For MMS user data uploading use gsm_upload mode() and use or modify function
     MMSPayloadCut to cut the upload data and sent only useful payload
  6) There is custom Poteron_Device_Reset function for the measure device restart.
  ==============================================================================

  */
#include <gsm_dal.h>

int MMSPayloadCut(gsm_device_t * module);

/*!
 * @brief  GSM_OK state -  idle state
 *
 * @param[in]     pcmd_val - pointer on the shared value of FSM
 *
 * @return        counter_required requirement to start command 1s timer
 *                in case of 0 = counter not required
 *                in case of 1 = counter is required
 *
 **/
int gsm_idle_state(gsm_device_t* module) {

	gsm_at_cmd_send(GSM_AT, 1);

	module->cmd_val = 0;
	module->tim_reqd = 1;

	return 0;
}

/*!
 *   @brief  GSM_SMS_RCVD -  new SMS received state
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *\n
 *\n Routine creates custom command to read SMS of certain number, note that
 *\n number was obtained using usual AT command enclosed to 'SM' RSPS_SM response
 *\n
 *
 **/
int gsm_SMS_tbread(gsm_device_t *module) {

	blue_led_mode_ctl(ONCE_FLASH);
	sprintf(module->cmd_to_send, "AT+CMGR=%ld", module->sms_num);
	gsm_at_cmd_send(module->cmd_to_send, 1);

	module->cmd_val = 0;
	module->tim_reqd = 1;

	return 0;
}

/*!
 *   @brief  GSM_SMS_READ -  Publish already read SMS
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n Clean up the raw SMS transaction to get clear text content of message
 *\n and publish this content using poteron_UART_tx()
 *\n
 *\n @poteron_UART_tx()
 *\n @gsm_read_sms_content()
 *\n
 *
 **/
int gsm_SMS_publish(gsm_device_t *module) {

	char * p_sms_message = NULL;

	p_sms_message = gsm_read_sms_content(module->p_rspd);
	/* Print SMS to UART*/
	poteron_UART_tx(p_sms_message);

	gsm_get_email_recipient(p_sms_message, module);
	/* Invoke deleting command as the consequent step*/
	module->cmd_val = GSM_SMS_DEL;
	memset(module->p_rspd, '\0', UART_BUFFSIZE);
	module->tim_reqd = 1;

	return 0;
}

/*!
 *   @brief  GSM_SMS_DEL - Delete SMS in the GSM module memory
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n Create custom command - of the certain SMS number to delete it from memory
 *\n
 *
 **/
int gsm_SMS_del(gsm_device_t *module) {

	sprintf(module->cmd_to_send, "AT+CMGD=%ld", module->sms_num);
	gsm_at_cmd_send(module->cmd_to_send, 1);

	blue_led_mode_ctl(NO_FLASH);
	module->cmd_val = 0;
	module->tim_reqd = 1;

	return 0;
}

/*!
 *   @brief  GSM_RING - Process module dialing - will cause as device HARD reset
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n If module is dialed by someone for dial_max cycles - system will
 *\n at the first pull down reset pin of Poteron Measure Device and then
 *\n will reset itself - should be used as remote system reset
 *\n
 *
 **/
int gsm_mod_dialed(gsm_device_t *module) {

	blue_led_mode_ctl(FAST_FLASH);
	/* May be optimized by user*/
	const uint8_t dial_max = 4;

	module->rst_beep = (module->rst_beep + 1) & (dial_max - 1);

	if (!module->rst_beep) {

		/* Blink with Blue LED and pull down RESET pin of Poteron Device*/
		HAL_GPIO_WritePin(N_POTERON_RST_GPIO_Port, N_POTERON_RST_Pin,
				GPIO_PIN_SET);
		HAL_Delay(5000);
		module->rst_req = 1;
	} else {
		gsm_at_cmd_send(GSM_AT, 1);
	}

	module->cmd_val = GSM_OK;
	/* Set the flag for reset*/

	return 0;
}

/*!
 *   @brief  Reset counter of dialed beep - tones and LED settle state
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n @sa gsm_mod_dialed()
 *\n
 **/
int gsm_mod_hangup(gsm_device_t *module)
{
	blue_led_mode_ctl(NO_FLASH);
	module->rst_beep = 0;
	module->cmd_val = GSM_OK;
	return 0;
}

/*!
 *   @brief  Routine to encapsule and send MMS
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        mms_state:
 *
 *                  GSM_MMS_PENDING - in case of successfully sent SMS (to BTS)
 *                  GSM_ERR - in case of any problem during encapsulation or transmitting
 *\n
 *\n Function obtain data to be send from the mms_content_buffer, if first character
 *\n of buffer is '\0', it is replaced by ' '. In case that result of strlen still 0
 *\n (first character still '\0') - string "ERR\n\r" is sent as MMS text data.
 *\n Function clears the content of mms_content_buffer each function call.
 *\n
 *\n The recipient of MMS is set using gsm_send_txt_mms()
 *\n
 *\n @gsm_send_txt_mms()
 *\n
 *
 **/
int gms_MMS_send(gsm_device_t *module) {

	int result = 0;
	int limited_resource = 0;
	/* For debug purposes - shows content of mms_content_buffer*/
	int res = 0;
	gsm_stop_MMS_DMA_UART(module);

	if(module->mms_pld[0] == '\0')
		module->mms_pld[0] = ' ';
	if(module->mms_pld[1] == '\0')
		module->mms_pld[1] = ' ';

	MMSPayloadCut(module);

	printf(module->mms_pld);

	/* Error call will be sent*/
	if (module->mms_pld == NULL) {
		module->mms_pld = RSPS_ERR;
		limited_resource = 1;
	}

	/* Send content of mms_content_buffer as MMS */
	res = gsm_send_txt_mms(module);

	/* Test If MMS routine was processed correctly*/
	if (res >= GSM_MMS_SENT) {
		module->cmd_val = GSM_OK;
	}
	else {
		module->rst_req = 1;
	}
	if (!limited_resource) {
		memset(module->mms_pld, '\0', strlen(module->mms_pld));
	}

	/* For debug purposes - shows result of gsm_send_txt_mms()*/
	printf("__MMS_ROUTINE_RESULT:%d__\r\n", res);

	/* Clear MMS_flg and enable ISR for further receiving data for MMS*/
	module->tim_reqd = 1;

	gsm_start_MMS_DMA_UART(module);

	return result;

}

/*!
 *   @brief  Function ties down and release pin dedicated for poteron RESET MCU pin
 *
 *   @param[in]     None
 *
 *   @return        None
 *
 *
 **/
void Poteron_Device_Reset(void) {
	/* Pulling down RST pin of processor on the Poteron Measure Device Board*/
	HAL_GPIO_WritePin(N_POTERON_RST_GPIO_Port, N_POTERON_RST_Pin, 0);
	HAL_Delay(500);
	HAL_GPIO_WritePin(N_POTERON_RST_GPIO_Port, N_POTERON_RST_Pin, 1);
	HAL_Delay(500);
}

/*!
 *   @brief  Routine to receive and process GSM module response
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *   @param[in]     print - control of printing/not printing
 *
 *   @return        0
 *
 *\n
 *\n Function is called when response from the module is received (preceded by sent
 *\n command) and response timer is in timeout state.
 *\n Received response is processed (optionally printed) and appropriate value of
 *\n pcmd_val is set using gsm_process_respond_fsm().
 *\n
 *\n Each function call are system timeout tim_gsm_timeout() and tim_gsm_respnd()
 *\n timers reseted.
 *\n
 *\n @sa gsm_process_respond_fsm()
 *\n
 **/

int gsm_rec_respnd2(gsm_device_t *module, uint8_t print) {


	if (print) {
		printf(module->p_rspd);
	}

	module->cmd_val = gsm_process_respond_fsm(module->p_rspd,
			BREAK_AT_GSM_SMS_READ);

	/* Extract SMS number from the command value (higher than GSM_SMS_RCVD and lower than 2*GSM_SMS_RCVD)*/
	if (module->cmd_val > GSM_SMS_RCVD && module->cmd_val <= GSM_SMS_RCVD * 2) {
		module->sms_num = module->cmd_val - GSM_SMS_RCVD;
		module->cmd_val = GSM_SMS_RCVD;
	}

	/* Clear buffer of received response, if its not needed for further usage (SMS publishing) */
	if (module->cmd_val == GSM_SMS_READ) {
		__NOP();
	} else {
		if(module->p_rspd != NULL)
			memset(module->p_rspd, '\0', UART_BUFFSIZE);
	}

	return 0;
}

/*!
 *   @brief  Function dedicated for no response, but response timeout took place
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n Function resets flag of mms_state from GSM_MMS_PENDING
 *\n and reset other flags for further communication and
 *\n prevent reset which could be triggered in gsm_rec_respond2()
 *\n
 *\n @gsm_rec_respond2()
 *\n
 **/
int gsm_missed_response(gsm_device_t *module) {
	const uint32_t max_attempts = 8u;

	green_led_mode_ctl(TWICE_FLASH);

	if (module->uart_attempt == 3) {
		printf("__MODULE_DOES_NOT_RESPOND__\r\n");

		gsm_uart_resolve_bugs();
		module->cmd_val = GSM_OK;
	}
	if (module->uart_attempt >= 3) {
		green_led_mode_ctl(FAST_FLASH);
	}
	module->uart_attempt = (module->uart_attempt + 1) & (max_attempts - 1);

	return 0;
}

/*!
 *   @brief  Function dedicated for uploading text content of the MMS
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0
 *
 *\n
 *\n Function reacts on the upload_mode set to GSM_MMS_POST value. Then measures time
 *\n until upload_timeout occurs and sets crucial flag cmd_val to GSM_MMS_POST value.
 *\n This should forward into the gms_MMS_send() routine to sent received content.
 *\n
 *\n @sa gms_MMS_send()
 *\n
 **/
int gsm_upload_mode(gsm_device_t *module) {
	static uint32_t local_time = 0u;
	static uint32_t current_time = 0u;
	const uint32_t upload_timeout = 20000u;

	if (module->upload_mode == GSM_MMS_POST) {

		local_time = HAL_GetTick();

		/* Time limit upload_timeout second for uploading*/
		if (local_time > current_time + upload_timeout || local_time < SECONDS_2) {
			/* Set the appropriate flags for MMS routine*/
			module->cmd_val = GSM_MMS_POST;
			/* Clear flag of the uploading*/
			module->upload_mode = GSM_NO_ACT;
		}

		/* Print the informational relative time in seconds of uploading*/
		if (((local_time - current_time) % 1000) == 0) {
			printf("__:%ld__\r\n", local_time >> 10);
		}
	} else {
		/* Get the time for upload routine timing - each iteration obtained*/
		current_time = HAL_GetTick();
	}

	return 0;
}

/*!
 *   @brief  Get useful begin and the end of the typical MMS message
 *
 *   @param[in]     p_module - pointer on instance of the gsm_device_t structure
 *
 *   @return        0 or -1 (Error - Borders had not been found)
 *
 *\n
 *\n @sa gms_MMS_send()
 *\n
 **/
int MMSPayloadCut(gsm_device_t * module)
{
	int lng = strlen(module->mms_pld);
	int idx = 0;

	for(idx = 0; idx < lng; idx++)
	{
		if(module->mms_pld[idx] == '<')
		{
			module->mms_pld = &(module->mms_pld[idx]);
			break;
		}
	}

	if(idx == lng - 1)
		return -1;

	lng = strlen(module->mms_pld);
	for(idx = lng - 1; idx >= 0; idx--)
	{
		if(module->mms_pld[idx] == '>')
		{
			memset(module->mms_pld+idx+1, '\0', lng - idx);
			break;
		}
	}
	if(idx == 0)
		return -1;

	return 0;
}
