/*
 * Quectel M95 - Mikroe GSM2 Click driver for STM32F373 driver
 *
 *  Created on: Nov 26, 2018
 *      Author: Vladimir Sustek
 */
/**
  @verbatim
  ==============================================================================
  How to use driver:

  0) Note that this file content except driver layer some extensions for SMS reading
     or MMS sending, not only hardware GSM module interfacing
  1) For the overall function is mandatory to customize UART3 functions (interface
     of the GSM module), whereas gsm_init_ring_DMA_UART(), refresh_uart3(),
     uart3_store_received() and gsm_uart_resolve_bugs() might be modified.
     The another stage - UART1 is dedicated only for interfacing special measure
     device, so might be absolutely unused, or completely modified. In this application
     is UART1 used to obtain data for MMS sending.
  2) Create a structure of the type gsm_device_t and set its cmd_val to GSM_OK and
     contact to any valid email address, the rest leave cleared to zeros or NULL
  3) Power-up GSM module (needed powerkey sequence) gsm_pwr_module_init(), warning
  4) Initialize system settings of the GSM using gsm_module_sys_init(), warning
     the current system settings is dedicated for the Czech VODAFONE MMS GATE !
  5) Send the appropriate command using gsm_send_cmd_ack_req2() or using the
     gsm_send_cmd_ack_req2(). Note that Initialization routine is content from
     these commands. Mentioned commands sends argument string to the module
     and awaits response while the time determined by the function is not elapsed.
     The response is hashed by gsm_process_respond_fsm() and compares known strings.
  6) For the MMS sending there is complete function gsm_send_txt_mms(), whereas
     is used second kind of data transmission to the module - gsm_at_data_send()
     This transmission only shifts data terminated by the 0x1A character and does
     not await any response such a gsm_send_cmd_ack_req2().
  7) The SMS message may be cleared and read using the gsm_read_sms_content, to use
     this function see its comment and implementation - crucial string operations
  8) The rest of the functions is more or less related to the described points, so
     should not be difficult to understand - this file may be divided to two layers.

  ==============================================================================

  */

/******************************************************************************
 * INCLUDES
 *****************************************************************************/
#include "gsm_driver.h"

/******************************************************************************
 * PUBLIC DATA
 *****************************************************************************/

/** List of commands dedicated for GSM module system init */
const char atok_command_list[16][50] = {
/******0123456789012345678901234567890101234567890123456789*/
/*00*/"AT+IPR=9600",
/*01*/"AT&W",
/*02*/"AT+QICSGP=1,\"CMWAP\"",
/*03*/"AT+QICSGP=1,\"mms\",\"mms\",\"mms\"",
/*04*/"AT+QMMURL=\"http://mms\"",
/*05*/"AT+QMMPROXY=1,\"10.11.10.111\",80",
/*06*/"AT+QFDEL=\"RAM:mms1.txt\"",
/*07*/"AT+QMMSCS=0",
/*08*/"AT+QMMSW=1,1,",
/*09*/"AT+QMMSCS=\"UTF8\",1",
/*10*/"AT+QMMSW=4,1",
/*11*/"AT+QMMSW=5,1,\"RAM:mms1.txt\"",
/*12*/"AT+QMMSEND=1",
/*13*/"AT+QFUPL=\"RAM:mms1.txt\"",
/*14*/"AT+QIDEACT",
/*15*/"AT" };

/** List of request commands dedicated for GSM read state */
const char atreq_command_list[3][16] = {
/******01234567890123456789012345678901*/
/*00*/"AT+CGREG?",
/*01*/"AT+CREG?",
/*02*/"AT+COPS?" };

/** List of expected successful responses from the module*/
const char at_response_list[15][16] = {
/******01234567890123456789012345678901*/
/*00*/"OK",
/*01*/"CREG",
/*02*/"CGREG",
/*03*/"RING",
/*04*/"SM",
/*05*/"Call Ready",
/*06*/"QMMNOTIFY",
/*07*/"CMGR",
/*08*/"NO CARRIER",
/*09*/"CME ERROR",
/*10*/"ERROR",
/*11*/"QFUPL",
/*12*/"CONNECT",
/*13*/"DEACT OK",
/*14*/"CMS ERROR" };

/******************************************************************************
 * PRIVATE DATA
 *****************************************************************************/

/******************************************************************************
 * Timers
 *****************************************************************************/

TIM_HandleTypeDef htim2;

static volatile uint32_t tim_2s_tmr_flag = 0;
static volatile uint32_t tim_2s_rst = 0;

static volatile uint32_t tim_0_5s_tmr_flag = 0;
static volatile uint32_t tim_0_5_rst = 0;

static volatile int green_led_mode = NO_FLASH;
static volatile int blue_led_mode = NO_FLASH;

/******************************************************************************
 * UART
 *****************************************************************************/

UART_HandleTypeDef huart1;
DMA_HandleTypeDef hdma_usart1_rx;
UART_HandleTypeDef huart3;
DMA_HandleTypeDef hdma_usart3_rx;

static char uart1_rx_buf[MMS_MAX_BUFF ];
static char uart3_rx_buf[UART_BUFFSIZE ];
static uint32_t uart3_rx_read_ptr = 0;

#define uart3_rx_write_ptr (UART_BUFFSIZE - hdma_usart3_rx.Instance->CNDTR)

volatile int uart3_received_flag = 0;

/* Shared between uart3_force_tx_pointer() and uart3_store_received() */
volatile static uint16_t idx_uart3 = 0;

/******************************************************************************
 * PUBLIC FUNCTIONS
 *****************************************************************************/

/*!
 * @brief  Pull 1/0 of power supply switch.
 *
 * @param[in]     state 0/1
 *
 * @return        state of the already set power supply pin.
 *
 *
 */
int gsm_pwr_supply_pull(int state) {
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_6, state);

	return HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_6);
}

/*!
 * @brief  Pull PWRKEY pin.
 *
 * @param[in]     state to be set
 *
 * @return        value of set pin - test if pin was set
 *
 *
 *\n PWRKEY pin is used to power-up gsm module
 *
 */
int gsm_pwrkey_pull(int state) {
	// should be PF7 on poteron
	HAL_GPIO_WritePin(GPIOF, GPIO_PIN_7, state);

	return HAL_GPIO_ReadPin(GPIOF, GPIO_PIN_7);
}

/*!
 * @brief  #RTS Pin pull.
 *
 * @param[in]     state to be set
 *
 * @return        value of set pin - test if pin was set
 *
 *
 *\n Emulates and controls #RTS Pin needed for GSM module UART stage
 *
 */
int gsm_nrts_pull(int state) {
	HAL_GPIO_WritePin(N_RTS_O_GPIO_Port, N_RTS_O_Pin, state);

	return HAL_GPIO_ReadPin(N_RTS_O_GPIO_Port, N_RTS_O_Pin);
}

/*!
 *   @brief  Handle Green_led_mode private variable of green_led blink routine
 *
 *   @param[in]     mode - Value to be stored into the green_led_mode
 *
 *   @return        green_led_mode - currently set value
 *
 *\n
 *\n Primitive function
 *\n
 **/
int green_led_mode_ctl(int mode) {
	green_led_mode = mode;
	return green_led_mode;
}

/*!
 *   @brief  Handle Blue_led_mode private variable of blue_led blink routine
 *
 *   @param[in]     mode - Value to be stored into the Blue_led_mode
 *
 *   @return        Blue_led_mode - currently set value
 *
 *\n
 *\n Primitive function
 *\n
 **/
int blue_led_mode_ctl(int mode) {
	blue_led_mode = mode;
	return blue_led_mode;
}

/*!
 *   @brief  Blink with LED in different modes
 *
 *   @param[in]     mode ONCE_FLASH, TWICE_FLASH, FAST_FLASH, NO_FLASH
 *
 *   @return        None
 *
 *\n
 *\n Function blinks with related led in specified intervals according to
 *\n the passed mode, see function implementation to get precise timing.
 *\n Number is statements should represent tens of miliseconds.
 *\n
 *\n Function is dedicated to be called periodically by timer 1x per 10ms
 *\n
 **/
void green_led_ctl(int mode) {
	/* LED LD1 flashing variable*/
	static uint16_t tim1_period = 0;

	if (mode < ONCE_FLASH) {
		return;
	}

	switch (tim1_period) {
	case 0: {
		HAL_GPIO_WritePin(Green_LED_GPIO_Port, Green_LED_Pin, 0);
		tim1_period++;
	}
		break;
	case 50: {
		HAL_GPIO_WritePin(Green_LED_GPIO_Port, Green_LED_Pin, 1);
		tim1_period++;
	}
		break;
	case 100: {
		if (mode == TWICE_FLASH || mode == FAST_FLASH)
			HAL_GPIO_WritePin(Green_LED_GPIO_Port, Green_LED_Pin, 0);
		if (mode == FAST_FLASH)
			tim1_period = 0;
		else
			tim1_period++;
	}
		break;
	case 150: {
		if (mode == TWICE_FLASH)
			HAL_GPIO_WritePin(Green_LED_GPIO_Port, Green_LED_Pin, 1);
		tim1_period++;
	}
		break;
	case 1000: {
		tim1_period = 0;
	}
		break;
	default: {
		tim1_period++;
	}
		break;
	}
}

/*!
 *   @brief  Blink with LED in different modes
 *
 *   @param[in]     mode ONCE_FLASH, TWICE_FLASH, FAST_FLASH, NO_FLASH
 *
 *   @return        None
 *
 *\n
 *\n Function blinks with related led in specified intervals according to
 *\n the passed mode, see function implementation to get precise timing.
 *\n Number is statements should represent tens of miliseconds.
 *\n
 *\n Function is dedicated to be called periodically by timer 1x per 10ms
 *\n
 **/
void blue_led_ctl(int mode) {
	/* LED LD1 flashing variable*/
	static uint16_t tim1_period = 0;

	if (mode < ONCE_FLASH) {
		if ((HAL_GPIO_ReadPin(Blue_LED_GPIO_Port, Blue_LED_Pin) != GPIO_PIN_SET))
			HAL_GPIO_WritePin(Blue_LED_GPIO_Port, Blue_LED_Pin, 1);
		return;
	}

	switch (tim1_period) {
	case 0: {
		HAL_GPIO_WritePin(Blue_LED_GPIO_Port, Blue_LED_Pin, 0);
		tim1_period++;
	}
		break;
	case 50: {
		HAL_GPIO_WritePin(Blue_LED_GPIO_Port, Blue_LED_Pin, 1);
		tim1_period++;
	}
		break;
	case 100: {
		if (mode == TWICE_FLASH || mode == FAST_FLASH)
			HAL_GPIO_WritePin(Blue_LED_GPIO_Port, Blue_LED_Pin, 0);
		if (mode == FAST_FLASH)
			tim1_period = 0;
		else
			tim1_period++;
	}
		break;
	case 150: {
		if (mode == TWICE_FLASH)
			HAL_GPIO_WritePin(Blue_LED_GPIO_Port, Blue_LED_Pin, 1);
		tim1_period++;
	}
		break;
	case 1000: {
		tim1_period = 0;
	}
		break;
	default: {
		tim1_period++;
	}
		break;
	}
}

/*!
 *   @brief  Start ring DMA UART stage UART3.
 *
 *   @param[in]     None
 *
 *   @return        0
 *
 */

int gsm_init_ring_DMA_UART(void) {
	HAL_UART_AbortReceive(&huart3);
	HAL_UART_Receive_DMA(&huart3, (uint8_t*) uart3_rx_buf, UART_BUFFSIZE);
	return 0;
}

/*!
 *   @brief  Stop ring DMA UART stage UART1.
 *
 *   @param[in]     None
 *
 *   @return        0
 *
 */
int gsm_stop_MMS_DMA_UART(gsm_device_t *module) {
	HAL_UART_AbortReceive(&huart1);
	module->mms_pld = uart1_rx_buf;
	return 0;
}
/*!
 *   @brief  Start ring DMA UART stage UART1.
 *
 *   @param[in]     None
 *w
 *   @return        0
 *
 */
int gsm_start_MMS_DMA_UART(gsm_device_t *module) {

	module->mms_pld = NULL;
	memset(uart1_rx_buf, '\0', MMS_MAX_BUFF);
	HAL_UART_Receive_DMA(&huart1, (uint8_t*) uart1_rx_buf, MMS_MAX_BUFF);
	return 0;
}
/*!
 *   @brief  Refresh content of DMA ring-UART buffer
 *
 *   @param[in]     p_module - pointer on the gsm_device_t structure element
 *
 *   @return        0
 *
 *
 */

int refresh_uart1(gsm_device_t* module) {

	static uint32_t uart1_cndtr = 0;
	uart1_cndtr = MMS_MAX_BUFF - hdma_usart1_rx.Instance->CNDTR;
	if (uart1_cndtr > 10) {
		module->upload_mode = GSM_MMS_POST;
	}
	return 0;
}

/*!
 *   @brief  Refresh content of DMA ring-UART buffer
 *
 *   @param[in]     p_module - pointer on the gsm_device_t structure element
 *
 *   @return        result - result of the UART refreshing :
 *                           -1 in case of aborted refresh (timeout UART_TO elapsed)
 *                           0  in case of successful refresh
 *
 *\n Function checks, if anything was received reading uart1_rx_write_ptr of the UART
 *\n whereas data are stored in the uart1_rx_buf array - global for the gsm_driver module
 *\n Consequently, each received character is stored using uart1_store_received into buffer
 *\n which is used for processing. In case of cyclic receiving longer than UART_TO, function
 *\n is aborted.
 *\n
 *\n Additionally, content of the second processing buffer is printed
 *\n
 *
 */

int refresh_uart3(gsm_device_t* module) {

	int result = 0;
	char rec_c = '\0';

	/* Supervising the routine using SysTick timer*/
	uint32_t acess_time;
	acess_time = HAL_GetTick();

	while (uart3_rx_read_ptr != uart3_rx_write_ptr) {
		if (HAL_GetTick() > acess_time + UART_TO || HAL_GetTick() < 5) {
			printf("Breaked UART3\r\n");
			result = -1;
			break;
		}
		__ISB();
		rec_c = uart3_rx_buf[uart3_rx_read_ptr];
		if (++uart3_rx_read_ptr >= UART_BUFFSIZE) {
			uart3_rx_read_ptr = 0;
		}
		module->p_rspd = uart3_store_received(rec_c, module);
	}
	return result;
}

/*!
 *   @brief  Store previously received character
 *
 *   @param[in]     recvd 	 	- received character to be stored
 *
 *   @param[in]     p_module 	- pointer on the gsm_device_t structure element
 *
 *
 *   @return        0
 *
 *\n Function checks, if anything was received reading uart1_rx_write_ptr of the UART
 *\n whereas data are stored in the uart1_rx_buf array - global for the gsm_driver module
 *\n Consequently, each received character is stored using uart1_store_received into buffer
 *\n which is used for processing. In case of cyclic receiving longer than UART_TO, function
 *\n is aborted.
 *\n
 *\n Additionally, content of the second processing buffer is printed
 *\n
 *
 */

int uart1_restore_buffer(void) {

	__ISB();
	memset(uart1_rx_buf, '\0', MMS_MAX_BUFF);
	return 0;
}

/*!
 *   @brief  Store previously received character
 *
 *   @param[in]     recvd 	 	- received character to be stored
 *
 *   @param[in]     p_module 	- pointer on the gsm_device_t structure element
 *
 *
 *   @return        p_ring_buffer_uart3 / NULL
 *
 *\n Function sequentially stores characters recvd until '\n' is stored - volatile flag
 *\n uart3_received_flag is set and this event is understood as end of the receiving.
 *\n During receiving index pointer variable idx_uart3 is incremented in module mode
 *\n to the maximal value equal to UART_BUFFSIZE - 1. The idx_uart3 can be forced to zero using
 *\n uart3_force_rx_pointer() or published using uart3_publish_pointer() functions. This
 *\n will cause overwrite of previously stored characters if receiving continues.
 *\n However, sometimes message contents more than one '\n' so it is better to wait
 *\n rather than trigger end of the message by this flag. Flag has to be cleared externally.
 *\n
 *\n Function can be accessed in special mode when both of the input paramters are 0 (NULL).
 *\n In this case, index idx_uart3 is set to zero and ring_buffer_uart3 array is erased to 0.
 *\n Special mode is simplified into the MACRO: UART1_CLEAN_BUFFERS()
 *\n
 *\n
 *
 */

char* uart3_store_received(char recvd, gsm_device_t* module) {

	char* p_rcvd = NULL;
	static char ring_buffer_uart3[UART_BUFFSIZE ];
	int respond_val = 0;

	ring_buffer_uart3[idx_uart3] = recvd;
	ring_buffer_uart3[(idx_uart3 + 1) & (UART_BUFFSIZE - 1)] = '\0';

	idx_uart3 = (idx_uart3 + 1) & (UART_BUFFSIZE - 1);

	if (recvd == '\n' || recvd == '>') {

		uart3_received_flag = 1;

		if (strlen(ring_buffer_uart3) == 0) {
			ring_buffer_uart3[0] = '"';
		}
		respond_val = gsm_process_respond_fsm(ring_buffer_uart3,
		DONT_BREAK_AT_GSM_SMS_READ);

		if (respond_val > GSM_SMS_RCVD && respond_val <= GSM_SMS_RCVD * 2) {
			idx_uart3 = 0;
		} else {
			switch (respond_val) {
			case GSM_OK :
			case GSM_SMS_RCVD :
			case GSM_RING :
			case GSM_CGREGOK :
			case GSM_CREGOK :
			case GSM_ERR :
			case GSM_CMEERR :
			case GSM_MMS_QFUPL :
			case GSM_CONNECT :
			case GSM_DEACTOK :
			case GSM_DATAUPLD :
			case GSM_CLLRDY :
			case GSM_CMSERR :
			case GSM_NOCRR : {
				idx_uart3 = 0;
			}
				break;
			}
		}

		p_rcvd = ring_buffer_uart3;
	}

	return p_rcvd;
}

/*!
 *   @brief  Restart UART stages UART1 and UART3.
 *
 *   @param[in]     None
 *
 *   @return        None
 *
 *\n After getting status of UARTs, is it Receving and Transmitting of UARTs
 *\n stoped and started again
 *
 *   @sa Used in main state machine of program.
 */

void gsm_uart_resolve_bugs(void) {
	HAL_UART_StateTypeDef st_1;
	uint32_t err_1 = 0;

	st_1 = HAL_UART_GetState(&huart3);

	err_1 = HAL_UART_GetError(&huart3);

	HAL_UART_AbortReceive(&huart3);
	HAL_UART_AbortTransmit(&huart3);

	gsm_init_ring_DMA_UART();

	printf("__UART3_RESTARTED__ST:%d__ERR%ld__\r\n", st_1, err_1);
}


/*!
 *   @brief  Print position of idx_uart3 index pointer related to the uart3_store_received().
 *
 *   @param[in]     None
 *
 *   @return        None
 *
 */

int uart3_publish_rx_pointer(void) {
	printf("__POINTER_UART3_POSITION: %d__\r\n", idx_uart3);
	return 0;
}

/*!
 *   @brief  Set position of the idx_uart3 index pointer of the uart3_store_received().
 *
 *   @param[in]     value
 *
 *   @return        idx_uart3 or -1
 *
 *\n
 *\n If value exceeds buffer limits <0; UART_BUFSIZE-1> -1 is returned and pointer unchanged
 *\n
 */

int uart3_force_rx_pointer(int value) {
	if (value > UART_BUFFSIZE - 1 || value < 0) {
		return -1;
	}

	idx_uart3 = value;

	return idx_uart3;
}

/*!
 * @brief  Sends AT command to UART3 (stage of GSM2 click module).
 *
 * @param[in]     pcmd             Char array of command to be sent .
 * @param[in]     appd_crlf        Append <CR><LF> to cmd array.
 *
 * @return         Status
 *                - -1       Invalid cmd parameter (NULL).
 *                - length of tx_o    Number Successfully sent characters.
 *
 *\n Request a blocking mode transmit of AT command (UART3) with possibility to
 *\n automatically apend <CR><LF> (GSM2 requires <CR><LF> in the end of command).
 *
 * @sa gsm_at_command().
 */
int gsm_at_cmd_send(char *cmd, int appd_crlf) {
	static char tx_o[UART_BUFFSIZE ];
	int result = 0;

	if ((cmd == NULL) || (strlen((char*) cmd) > UART_BUFFSIZE )) {
		return -1;
	} else {
		if (appd_crlf) {
			sprintf(tx_o, "%s\r\n", cmd);
		} else {
			sprintf(tx_o, "%s", cmd);
		}

		gsm_nrts_pull(0);
		result = strlen((char*) tx_o);

		HAL_UART_Transmit(&huart3, (uint8_t*) tx_o, strlen(tx_o),
		HAL_MAX_DELAY);

	}

	return result;
}
/*!
 * @brief  Sends and divides data to max 500 byte blocks to UART 2 (GSM 2 click).
 *
 * @param[in]     pData   Char array of data to be sent.
 * @param[in]     lng     lng of the command array of to be sent.
 *
 * @return        err     -Return number of errors during transmission.
 *
 *
 *\n At the first, Data is divided into the 500 bytes blocks, each block is sent
 *\n and 750 ms timeout should ensure correct Data processing of module (GSM Echo listened)
 *\n
 *
 * @sa gsm_send_txt_mms().
 */
int gsm_at_data_send(char *data, int lng) {
	/* Number of 500 - byte slots to be send*/
	int sub_trx = 0;
	/* Number of slot to be send*/
	int trx_idx = 0;
	/* Number of missed ACKs*/
	int err = 0;

	/* Data are divided into 500 Bytes slots to be send - calculate number of slots*/
	sub_trx = lng / 500;

	/* While all of slots are not send or amount of data is smaller than 500 bytes*/
	while (sub_trx != trx_idx) {
		/* Send the first slot*/
		HAL_UART_Transmit(&huart3, (uint8_t*) data + (500 * trx_idx), 500,
		HAL_MAX_DELAY);

		/* Increment the number of the slot to be send*/
		trx_idx++;

	}

	/* Transmit the last (or first in case of amount of data lower than 500) not full slot*/
	HAL_UART_Transmit(&huart3, (uint8_t*) data + (500 * trx_idx),
			lng - 500 * trx_idx, HAL_MAX_DELAY);

	/* Terminate Data transaction using expected SUB character '0x1A' */
	gsm_send_at_SUB();

	return err;
}

/*!
 * @brief  Sends SUB charater to terminate data transmission - Blocking Function.
 *
 * @param[in]     None
 *
 * @return        None
 *
 *
 *\n Sends 'SUB' = 0x1A character (UNIX system CTRL+Z) = stop character to terminate
 *\n data transmission.
 *
 * @sa gsm_at_data_send().
 */
void gsm_send_at_SUB(void) {
	static char sub_ch = 0x1A;

	HAL_UART_Transmit(&huart3, (uint8_t*) &sub_ch, 1u, HAL_MAX_DELAY);
}

/*!
 * @brief  Send AT command and wait for reply (until 500ms elapses).
 *
 * @param[in]     p_cmd       Pointer on the string to be sent as a command
 *
 * @param[in]     p_module    Pointer on the instance of the gsm_device_t type
 *
 *
 * @return        cmd_val in case of parsed response, or GSM_ERR in case of
 *                timeout (module had not responded)
 *
 *
 *\n
 *\n Functions uses function gsm_at_cmd_send() to send command and starts
 *\n tim_cmd_limit() timer (500ms). While the timer runs, UART is refreshed
 *\n using refresh_uart3() function and if appropriate uart3_received_flag had
 *\n been set ('\n' character received - valid response from the module), the
 *\n response is hashed using gsm_process_respond_fsm() to distinguish it.
 *\n Consequently, the content of UART buffer is cleared and only hased value
 *\n of response returned. In case of timeout - uart3_received_flag had not been
 *\n set, function returns GSM_ERR and clears current content of UART and timer.
 *\n
 * @sa gsm_at_cmd_send()
 * @sa tim_cmd_limit()
 * @sa uart3_received_flag
 * @sa gsm_process_respond_fsm()
 *
 */

int gsm_send_cmd_ack_req2(const char *cmd, gsm_device_t *module) {
	/* Send command and always append <CR><LF> */
	gsm_at_cmd_send((char*) cmd, 1);

	/* Start the timer - blocking waiting for response*/
	tim_cmd_limit(START_THE_TIMER);

	while (!tim_cmd_limit(GET_TIMER_STATE)) {
		refresh_uart3(module);
	}

	if (uart3_received_flag == 0) {
		return GSM_ERR ;

	}

	/* Process received response*/
	module->cmd_val = gsm_process_respond_fsm(module->p_rspd,
	BREAK_AT_GSM_SMS_READ);

	if (module->p_rspd != NULL) {
		printf(module->p_rspd);
		memset(module->p_rspd, '\0', UART_BUFFSIZE);
	}

	/* Prepare timer for another function call*/
	tim_cmd_limit(RESET_THE_TIMER);

	/* Clear UART3 ISR */
	uart3_received_flag = 0;

	return module->cmd_val;
}

/*!
 * @brief  Send AT command and wait for reply (until 2s elapses).
 *
 * @param[in]     p_cmd       Pointer on the string to be sent as a command
 *
 * @param[in]     p_module    Pointer on the instance of the gsm_device_t type
 *
 *
 * @return        cmd_val in case of parsed response, or GSM_ERR in case of
 *                timeout (module had not responded)
 *
 *
 *\n
 *\n Functions uses function gsm_at_cmd_send() to send command and starts
 *\n tim_gsm_respnd() timer (2s). While the timer runs, UART is refreshed
 *\n using refresh_uart3() function and if appropriate uart3_received_flag had
 *\n been set ('\n' character received - valid response from the module), the
 *\n response is hashed using gsm_process_respond_fsm() to distinguish it.
 *\n Consequently, the content of UART buffer is cleared and only hased value
 *\n of response returned. In case of timeout - uart3_received_flag had not been
 *\n set, function returns GSM_ERR and clears current content of UART and timer.
 *\n
 * @sa gsm_at_cmd_send()
 * @sa tim_gsm_respnd()
 * @sa uart3_received_flag
 * @sa gsm_process_respond_fsm()
 *
 */

int gsm_send_cmd_ack_req3(const char *cmd, gsm_device_t *module) {

	tim_gsm_respnd(RESET_THE_TIMER);

	/* Send command and always append <CR><LF> */
	gsm_at_cmd_send((char*) cmd, 1);

	/* Start the timer - blocking waiting for response*/
	tim_gsm_respnd(START_THE_TIMER);

	while (tim_gsm_respnd(GET_TIMER_STATE) == 0) {
		refresh_uart3(module);
	}

	if (uart3_received_flag == 0) {
		/* Optional - print what was received in previous step*/
		return GSM_ERR ;
	}

	uart3_received_flag = 0;

	/* Process received response*/
	module->cmd_val = gsm_process_respond_fsm(module->p_rspd,
	BREAK_AT_GSM_SMS_READ);

	if (module->p_rspd != NULL) {
		printf(module->p_rspd);
		memset(module->p_rspd, '\0', UART_BUFFSIZE);

	}

	/* Prepare timer for another function call*/
	tim_gsm_respnd(RESET_THE_TIMER);

	/* Clear UART3 ISR */
	uart3_received_flag = 0;

	return module->cmd_val;
}

/*!
 * @brief  Function linked with standard output - uses implemented UART as printf()
 *
 * @return        amount of printed characters
 *
 */
int _write(int file, char const *buf, int n) {
	/* stdout redirection to UART2 */
	HAL_UART_Transmit(&huart2, (uint8_t*) (buf), n, HAL_MAX_DELAY);
	return n;
}

/*!
 * @brief  Function similar to printf(), prints to implemented UART stage
 *
 * @param[in]     p_str       string or pointer on the string to be printed
 *
 *
 * @return       0
 *
 */
int poteron_UART_tx(const char *str) {
	int result = 0;

	HAL_UART_Transmit(&huart1, (uint8_t*) str, strlen(str), HAL_MAX_DELAY);

	return result;
}

/*!
 * @brief  Function initialize members of gsm_device_t structure to default for start
 *
 * @param[in]     p_module    Pointer on the instance of the gsm_device_t type
 *
 * @return        0
 *
 */
int gsm_init_module_struct(gsm_device_t* module) {

	module->cmd_val = GSM_OK;
	module->p_rspd = NULL;
	module->sms_num = GSM_NO_ACT;
	module->tim_reqd = GSM_NO_ACT;
	module->rst_req = GSM_NO_ACT;
	module->mms_pld = NULL;
	module->upload_mode = GSM_NO_ACT;
	module->uart_attempt = GSM_NO_ACT;
	module->rst_beep = 0;
	strcpy(module->contact, "vladimir.sustek@sobriety.cz");
	memset(module->cmd_to_send, '\0', UART_BUFFSIZE);

	return 0;
}

/*!
 * @brief  Power up the gsm module 'GSM2 click' using PWRKEY.
 *
 * @param[in]     None.
 * @param[in]     None.
 *
 * @return        None.
 *
 *\n Function takes around 16 seconds to be done. Sensing of STA pin can
 *\n reduce function time (pulled after power up by PWRKEY). At the first
 *\n measure device (intended for Poteron) is pulled and power supply switch
 *\n of the module is turned on. Consequently, recommended power up sequence
 *\n of the Quectel M95 module is processed.
 *
 */
void gsm_pwr_module_init(void) {
	HAL_GPIO_WritePin(N_POTERON_RST_GPIO_Port, N_POTERON_RST_Pin, 0);
	gsm_pwr_supply_pull(0);
	HAL_Delay(1000);
	HAL_GPIO_WritePin(N_POTERON_RST_GPIO_Port, N_POTERON_RST_Pin, 1);
	gsm_pwr_supply_pull(1);
	HAL_Delay(1000);
	gsm_pwrkey_pull(0);
	HAL_Delay(100);
	gsm_pwrkey_pull(1);
	HAL_Delay(2500);
	gsm_pwrkey_pull(0);
	HAL_Delay(13000);
}

/*!
 * @brief  Initialize system and set network parameters (GSM, GPRDS)
 *
 * @param[in]     p_module    Pointer on the instance of the gsm_device_t type
 *
 * @return       cmd_rsps_id:
 *
 *               -1 failed to reach ready state - module does not respond
 *               -2 failed to evaluate network registration
 *
 *               positive number equal to deleted SMS (passed)
 *\n
 *\n (1) Wait until is module not ready
 *\n (2) Change settings of module:
 *\n      [0] Set Baud Rate
 *\n      [1] Save Settings
 *\n      [2] Set APN mode 1
 *\n      [3] Set APN mode 2
 *\n      [4] Set MMS gate
 *\n      [5] Set Proxy Server
 *\n (3) Delete SMS to clear memory for further receiving
 *\n (4) Check network registration GPS and GPRS
 *\n
 */

int gsm_module_sys_init(gsm_device_t *module) {
	int cmd_sent = 0;
	int cmd_rsps_id = 0;
	char custom_cmd[16] = { "\0" };

	/* Test module using empty AT commands: pass or fail */
	while (cmd_sent < GSM_INIT ) {
		int sub_rsps_id = 0;
		sub_rsps_id = gsm_send_cmd_ack_req2(GSM_AT, module);
		cmd_rsps_id += sub_rsps_id;
		cmd_sent++;
		if (sub_rsps_id == GSM_CLLRDY)
			break;
	}

	if (cmd_rsps_id < GSM_INIT / 2) {
		return -1;
	}
#warning The following settings is dedicated for Vodafone MMS gate
	/* Send first 6 commands of settings to use module for
	 *  the MMS sending and the SMS receiving (Vodafone MMS):
	 * [0] Set Baud Rate
	 * [1] Save Settings
	 * [2] Set APN mode 1
	 * [3] Set APN mode 2
	 * [4] Set MMS gate
	 * [5] Set Proxy Server
	 */
	cmd_rsps_id = 0;
	cmd_sent = 0;

	while (cmd_sent < 6) {
		cmd_rsps_id += gsm_send_cmd_ack_req2(atok_command_list[cmd_sent],
				module);
		cmd_sent++;
	}

	/* Delete received SMS for further usage of SMS receive memory
	 * the amount of deleted SMS is set by SMS_TO_DELETE*/

	cmd_sent = 1;
	cmd_rsps_id = 0;

	while (cmd_sent < SMS_MAX_CNT / 4) {
		sprintf(custom_cmd, "AT+CMGD=%d", cmd_sent);
		cmd_rsps_id += gsm_send_cmd_ack_req2(custom_cmd, module);
		cmd_sent++;
	}

	/* Test if the module is registered in GSM and GPRS network by:
	 * [0] Test of GPRS	(CGREG)
	 * [1] Test of GSM	(CREG)
	 * [2] Test of network registration
	 */

	cmd_sent = 2;
	while (cmd_sent < 3) {
		cmd_rsps_id += gsm_send_cmd_ack_req2(atreq_command_list[cmd_sent],
				module);
		cmd_sent++;
	}

	return cmd_rsps_id;
}

/*!
 * @brief  Parse and has received response of the Quectel M95 module
 *
 * @param[in]    p_str    pointer on the string to be parsed and hashed
 *
 * @param[in]    break_at_GSM_SMS_READ  break control of SMS read response
 *
 * @return       result   hashed value of response
 *\n
 *\n Function tokenize passed string until any key - string is found or nothing
 *\n had been found. If string was found function returns its value except low
 *\n priority RSPS_OK = GSM_OK case.
 *\n
 */
int gsm_process_respond_fsm(char* str, int break_at_GSM_SMS_READ) {
	/* Variables of strtok_r for tokenizing */
	char bckp[UART_BUFFSIZE ]; /* Back-up array to do not change original p_str content*/
	memset(bckp, '\0', UART_BUFFSIZE);
	memcpy(bckp, str, strlen(str));

	char *rest = NULL;
	char *token;
	char dlmtr[] = { ",+:\n\r\"\"" }; /* Delimiters for tokenizing */

	int result = 0;

	/** Tokenizing loop - string is divided into tokens according to the delimiters dlmtr
	 * */
	for (token = strtok_r(bckp, dlmtr, &rest); token != NULL;
			token = strtok_r(NULL, dlmtr, &rest)) {

		/** Detect if module is dialed by someone*/
		if (!(strcmp(token, RSPS_RING))) {
			result = GSM_RING;
			break;
		}
		/** Detect if module is dialing ended up*/
		if (!(strcmp(token, RSPS_NOCRR))) {
			result = GSM_NOCRR;
			break;
		}

		/** Detect if module is registered in GSM network */
		if (!(strcmp(token, RSPS_CREG))) {
			/** Forced next iteration of tokenizing to get 1st value of CREG*/
			token = strtok_r(NULL, dlmtr, &rest);
			if (!(strcmp(token, " 0"))) {
				/** Forced next iteration of tokenizing to get 2nd value of CREG*/
				token = strtok_r(NULL, dlmtr, &rest);
				if (!(strcmp(token, "1"))) {
					/* Module is registered in GSM network = GREG 01*/
					result = GSM_CREGOK;
					break;
				}
			}
		}

		/** Detect if module is registered in GPRS network */
		if (!(strcmp(token, RSPS_CGREG))) {
			/** Forced next iteration of tokenizing to get 1st value of CGREG*/
			token = strtok_r(NULL, dlmtr, &rest);
			if (!(strcmp(token, " 0"))) {
				/** Forced next iteration of tokenizing to get 2nd value of CGREG*/
				token = strtok_r(NULL, dlmtr, &rest);
				if (!(strcmp(token, "1"))) {
					/* Module is registered in GPRS network = GGREG 01*/
					result = GSM_CGREGOK;
					break;
				}
			}
		}

		/** Empty command echo detection - module is alive in idle state */
		if (!(strcmp(token, RSPS_OK))) {
			result = GSM_OK;
		}

		/** Detect if modem is call ready - related to RSPS_CGREG and RSPS_CREG, can be
		 * dialed or can send something, however RSPS_CGREG + RSPS_CGREG should be done
		 * */
		if (!(strcmp(token, RSPS_CLLRDY))) {
			result = GSM_CLLRDY;
			break;
		}

		/** Detect if MMS was send and acknowledge of network was received successfully*/
		if (!(strcmp(token, RSPS_GGMNTF))) {
			result = GSM_MMS_SENT;
			break;
		}

		/* If reading of SMS is in progress = awaited content of sms to be printed*/
		if (!(memcmp(token, RSPS_SMSRD, 4))) {
			result = GSM_SMS_READ;

			if (break_at_GSM_SMS_READ) {
				break;
			}
		}

		/** Detect if modem received SMS, tokenizing contents forced next iteration
		 * to get number of the SMS for further processing (read this sms)
		 * */
		if (!(strcmp(token, RSPS_SM))) {
			/* Get the number of the SMS - follows 'SM' flag*/
			token = strtok_r(NULL, dlmtr, &rest);
			result = GSM_SMS_RCVD + strtol(token, &rest, CONVERT_10_BASE);
			break;

		}
		/** If reading of CME ERROR occured*/
		if (!(strcmp(token, RSPS_CMEERR))) {
			result = GSM_CMEERR;
			break;
		}

		/** If general error had been received*/
		if (!(strcmp(token, RSPS_ERR))) {
			result = GSM_ERR;
			break;
		}

		/** If  QFUPL is in progress (uploading MMS content)*/
		if (!(strcmp(token, RSPS_UPL))) {
			result = GSM_MMS_QFUPL;
			break;
		}

		/** If  CMS ERROR occured (Errors related to MMS sending and handling)*/
		if (!(strcmp(token, RSPS_CMSERR))) {
			result = GSM_CMSERR;
			break;
		}

		/** If  module is in uploading mode - connect and listening UART line*/
		if (!(strcmp(token, RSPS_CONNECT))) {
			result = GSM_CONNECT;
			break;
		}

		/** If GPRS context was deactived successfully - related to MMS send routine*/
		if (!(strcmp(token, RSPS_DEACTOK))) {
			result = GSM_DEACTOK;
			break;
		}

		/** If module is in short upload mode - e.g. receiving MMS title or SMS content*/
		if (strchr(token, '>') != NULL) {
			result = GSM_DATAUPLD;
			break;
		}

	}

	return result;
}

/*!
 * @brief  Prepares and sends mms to the fixed address passed into the function.
 *
 * @param[in]     p_module    Pointer on the instance of the gsm_device_t type
 *
 * @return        cmd_val    Result of all used function to invoke MMS send
 *                           awaited 8 in case of successful transmission
 *
 *
 * Correct number or e-mail address is expected, Data nor Contact can be empty array (NULL)
 *
 * Sending routine is content from following steps and delay steps such an empty AT commands or timers (see implementation):
 *
 * 1) Attempt to delete RAM (if MMS was already sent in the 'program life'
 *
 * 2) Shift data to be sent into the RAM
 *
 * 3) Clear MMS handle content
 *
 * 4) Create new MMS handle - add recipient (contact)
 *
 * 5) Change encoding to UTF - 8
 *
 * 6) Add MMS title (part of MMS handle)
 *
 * 7) Attach uploaded file in step 2 to the MMS handle
 *
 * 8) Send MMS
 *
 * 9) Wait until MMS is sent
 *
 * 10) Deactive GPRS context
 *
 *\n
 *
 */
int gsm_send_txt_mms(gsm_device_t *module) {

	printf("MMS\r\n");
	char custom_command[BUFSIZ / 4] = { "\0" };

	int cmd_val = 0;

	int data_lng = 0;

	const char MMS_title[] = { "PoteronDataMsg" };

	if (module->mms_pld == NULL || module->contact == NULL) {
		return -1;
	}
	/* (1.0) Try to delete data if any was stored before */
	cmd_val += gsm_send_cmd_ack_req3(GSM_QDEL_DATA_TXT, module);

	/* (2.0) Calculate real length of data to be sent */
	data_lng = strlen(module->mms_pld);
	/* (2.1) Create command to upload data to be sent */
	sprintf(custom_command, "%s, %d", GSM_QFUPL, data_lng);
	/* (2.2) Upload data - send appropriate command */
	cmd_val += gsm_send_cmd_ack_req3(custom_command, module);
	/* (2.3) Shift data into the module - upload data */
	gsm_at_data_send(module->mms_pld, strlen(module->mms_pld));
	tim_gsm_respnd(START_THE_TIMER);
	while (tim_gsm_respnd(GET_TIMER_STATE) == 0) {
		refresh_uart3(module);
	}
	printf(module->p_rspd);
	tim_gsm_respnd(RESET_THE_TIMER);

	/* (3.0) Clear MMS handle content*/
	cmd_val += gsm_send_cmd_ack_req3("AT+QMMSW=0", module);

	/* (4.0) Add recipient to receive MMS - command create*/
	sprintf(custom_command, "%s\"%s\"", GSM_QMMSSW_QMM_1, module->contact);
	/* (4.1) Send command to add certain recepient */
	cmd_val += gsm_send_cmd_ack_req3(custom_command, module);
	CLEAR_WATCH_DOG();

	/* (5.0) Set encoding to the UTF8 */
	cmd_val += gsm_send_cmd_ack_req3(GSM_QMMSCS_UTF8, module);

	/* (6.0) Add title of MMS - set adding command */
	cmd_val += gsm_send_cmd_ack_req3(GSM_QMMSW_QMM_4, module);
	/* (6.1) Shift data into the module */
	gsm_at_data_send((char*) MMS_title, strlen(MMS_title));

	tim_gsm_respnd(START_THE_TIMER);
	while (tim_gsm_respnd(GET_TIMER_STATE) == 0) {
		refresh_uart3(module);
	}
	printf(module->p_rspd);
	tim_gsm_respnd(RESET_THE_TIMER);

	/* (7.0) Attach previously uploaded file into the MMS handle */
	cmd_val += gsm_send_cmd_ack_req3(GSM_QMMSW_QMM_5, module);

	CLEAR_WATCH_DOG();

	/* (8.0) Send the MMS handler - send MMS */
	cmd_val += gsm_send_cmd_ack_req3(GSM_QMMSEND, module);

	/* (9.0) Wait until MMS is sent (GSM_MMS_SENT response is recevied) */
	uint32_t waiting_cycle = 1;
	while (waiting_cycle <= GSM_MMS_TO_MAX ) {
		/* Each 5th Empty command refresh WATCH_DOG*/
		if (waiting_cycle % 4 == 0) {
			CLEAR_WATCH_DOG();
		}
		/* If GSM_MMS_SENT response processed = GMMNOTIFY*/
		if (cmd_val > GSM_MMS_SENT) {
			break;
		}
		cmd_val += gsm_send_cmd_ack_req3(GSM_AT, module);
		waiting_cycle++;
	}
	if (waiting_cycle == GSM_MMS_TO_MAX) {
		module->rst_req = 1;
	}
	uart3_force_rx_pointer(GSM_NULL);

	/* (10.0) Deactive GPRS context*/
	cmd_val += gsm_send_cmd_ack_req3(GSM_QIDEACT, module);
	tim_gsm_respnd(START_THE_TIMER);
	while (tim_gsm_respnd(GET_TIMER_STATE) == 0) {
		refresh_uart3(module);
	}
	printf(module->p_rspd);
	tim_gsm_respnd(RESET_THE_TIMER);

	return cmd_val;
	/* Return value which indicates returned values of sent commands - can check */
}

/*!
 * @brief  Obtain email or number of the MMS from the received SMS.
 *
 * @param[in]     p_str       Pointer on the SMS content string array.
 * @param[in]     p_module    Pointer on the instance of gsm_device_t type structure
 *
 * @return        0
 *
 *\n
 *\n Function tokenizes p_str until finds '@' character. Then searching for borders
 *\n of the email until ' ' spaces are found on the left and right beside the '@'.
 *\n Found email is copied into the contact member of the passed input structure.
 *\n
 *\n Also is function able to extract Czech number (+420123456789)
 *
 */
int gsm_get_email_recipient(char *sms_content, gsm_device_t *module) {
	char bckp[UART_BUFFSIZE ];
	memset(bckp, '\0', UART_BUFFSIZE);
	memcpy(bckp, sms_content, strlen(sms_content));

	char *rest = NULL;
	char *token;
	char dlmtr[] = { " ,+:\n\r\"\"" };

	/** Tokenizing loop - string is divided into tokens according to the delimiters dlmtr
	 * */
	for (token = strtok_r(bckp, dlmtr, &rest); token != NULL;
			token = strtok_r(NULL, dlmtr, &rest)) {
		if (strchr(token, '@') != NULL) {
			printf("__RECIPIENT_CHANGED__ ");
			printf(token);
			printf("\r\n");
			strcpy(module->contact, token);
			break;
		}
		/* Czech Number */
		if (memcmp(token, "420", 3)==0) {
			printf("__RECIPIENT_CHANGED__ ");
			printf(token);
			printf("\r\n");
			strcpy(module->contact+1, token);
			module->contact[0] = '+';
			break;
		}
	}

	return 0;
}

/*!
 * @brief  Obtain number of content from the receiving sms.
 *
 * @param[in]     str    Pointer on the SMS content string array.
 *
 * @return        pointer on the being of SMS text content or NULL in case of error
 *
 *
 *\n
 *\n Function is looking for last '"' character, which is actualy the last
 *\n part of SMS header, then only \r \n chars are before clear SMS content.
 *\n Consequently, the tail of the SMS is cut off using '\0' characters.
 *\n Warning for SMS which content '"' character - function may be misused
 *\n
 *\n
 *\n "AT+CMGR=1\r\r\n+CMGR: \"REC UNREAD\",\"+420773920424\",\"\",\"2018/10/29
 *\n 08:40:45+04\"\r\nmr 3 mrhn_0_1 1 2 2 2 1 0 0 0\r\n\r\nOK\r\n"
 *\n
 *
 */
char* gsm_read_sms_content(char * str) {
	/* Position of the '"' */
	size_t pos = 0;
	/* Delimiter '"' */
#warning SMS must not contain '"' character as payload
	const char dlmt = '"';

	/* Character compared with delimiter*/
	char rx_c = 0;
	size_t lng = 0;
	static char *p_res = NULL;
	p_res = str;

	/* Error detection in case of overflowed array*/

	if ((pos = strlen(p_res)) > UART_BUFFSIZE) {
		return NULL;
	} else {
		/* Analyze the array until delimiter is not found or
		 * 2nd member of array is reached - prevent fault*/

		while (pos != 1) {
			/* Stop search in case of hit delimiter*/
			rx_c = p_res[pos - 1];
			if (rx_c == dlmt) {
				break;
			}
			pos--;
		}

		/* Return pointer located in the array, whereas SMS text content begin */
		p_res = &(str[pos + 2]);
	}

	lng = strlen(p_res);
	pos = lng - 1;

	while (pos >= lng - 10) {
		if (!memcmp(p_res + pos, "AT", 2)) {
			p_res[pos] = '\0';
		}
		if (!memcmp(p_res + pos, "OK", 2)) {
			p_res[pos] = '\0';
		}
		pos--;
	}

	return p_res;
}

/*!
 *   @brief  Counter for 2 second timer
 *
 *   @param[in]     None
 *
 *   @return        None
 *
 *\n
 *\n Function increments timer value, or resets timer
 *\n according to the set flag related to the timer
 *\n
 **/
void timer_2_second(void) {
	if (tim_2s_rst == 1) {
		tim_2s_tmr_flag = 0;
		tim_2s_rst = 0;
	}
	switch (tim_2s_tmr_flag) {
	case 0: {
	}
		break;
	case SECONDS_2 : {
	}
		break;
	default: {
		tim_2s_tmr_flag++;
	}
		break;
	}
}

/*!
 *   @brief  Counter for 500ms timer
 *
 *   @param[in]     None
 *
 *   @return        None
 *
 *\n
 *\n Function increments timer value, or resets timer
 *\n according to the set flag related to the timer
 *\n
 **/
void timer_500miliseconds(void) {
	if (tim_0_5_rst == 1) {
		tim_0_5s_tmr_flag = 0;
		tim_0_5_rst = 0;
	}
	switch (tim_0_5s_tmr_flag) {
	case 0: {
	}
		break;
	case SECOND_0_5 : {
	}
		break;
	default: {
		tim_0_5s_tmr_flag++;
	}
		break;
	}
}

/*!
 *   @brief  500 milisecond timer
 *
 *   @param[in]     arg - START_THE_TIMER, RESET_THE_TIMER, GET_TIMER_STATE
 *
 *   @return        result : 1 in case of elapsed time period, otherwise 0
 *
 *\n
 *\n Function allows to at the first start timer by START_THE_TIMER, get the
 *\n state using GET_TIMER_STATE and clear timer for further usage using
 *\n RESET_THE_TIMER. Timer is reseted synchronously in general timer callback.
 *\n
 **/
int tim_cmd_limit(int arg) {
	int result = 0;

	switch (arg) {
	case START_THE_TIMER : {
		if (tim_0_5s_tmr_flag == 0) {
			tim_0_5s_tmr_flag = 1;
		}

	}
		break;
	case RESET_THE_TIMER : {
		if (tim_0_5s_tmr_flag >= SECOND_0_5) {
			tim_0_5s_tmr_flag = 0;
		} else {
			tim_0_5_rst = 1;
		}

	}
		break;
	case GET_TIMER_STATE : {
		if (tim_0_5s_tmr_flag >= SECOND_0_5) {
			result = 1;

		}
	}
		break;
	}

	return result;
}
/*!
 *   @brief  2 second timer
 *
 *   @param[in]     arg - START_THE_TIMER, RESET_THE_TIMER, GET_TIMER_STATE
 *
 *   @return        result : 1 in case of elapsed time period, otherwise 0
 *
 *\n
 *\n Function allows to at the first start timer by START_THE_TIMER, get the
 *\n state using GET_TIMER_STATE and clear timer for further usage using
 *\n RESET_THE_TIMER. Timer is reseted synchronously in general timer callback.
 *\n
 **/
int tim_gsm_respnd(int arg) {
	int result = 0;
	switch (arg) {
	case START_THE_TIMER : {
		if (tim_2s_tmr_flag == 0) {
			tim_2s_tmr_flag = 1;
		}

	}
		break;
	case RESET_THE_TIMER : {
		if (tim_2s_tmr_flag >= SECONDS_2) {
			tim_2s_tmr_flag = 0;
		} else {
			tim_2s_rst = 1;
		}
	}
		break;
	case GET_TIMER_STATE : {
		if (tim_2s_tmr_flag >= SECONDS_2) {
			result = 1;
		}
	}
		break;
	}

	return result;
}

/*!
 *   @brief  Implementation of the HAL_UART_ErrorCallback() function
 *           with implemented UART1-3 to solve errors of the UART3
 *           and UART1 during DMA receiving. UARTs are in case of
 *           error completely restarted and state + error printed.
 *
 */
void HAL_UART_ErrorCallback(UART_HandleTypeDef *huart) {
	if (huart->Instance == USART3) {
		HAL_UART_AbortTransmit(&huart3);
		printf("__USART3_ST:%x__\t", HAL_UART_GetState(&huart3));
		printf("__USART3_ER:%lx__\r\n", HAL_UART_GetError(&huart3));

		HAL_UART_AbortReceive(&huart3);
		HAL_UART_Receive_DMA(&huart3, (uint8_t*) uart3_rx_buf, UART_BUFFSIZE);
	}
	if (huart->Instance == USART2) {
		HAL_UART_AbortTransmit(&huart2);
		printf("__USART2_ST:%x__\t", HAL_UART_GetState(&huart2));
		printf("__USART2_ER:%lx__\r\n", HAL_UART_GetError(&huart2));
	}
	if (huart->Instance == USART1) {
		HAL_UART_AbortTransmit(&huart1);
		printf("__USART1_ST:%x__\t", HAL_UART_GetState(&huart3));
		printf("__USART1_ER:%lx__\r\n", HAL_UART_GetError(&huart3));

		HAL_UART_AbortReceive(&huart1);
		HAL_UART_Receive_DMA(&huart1, (uint8_t*) uart1_rx_buf, MMS_MAX_BUFF);
	}

}

/*!
 *   @brief  Implementation of the HAL_TIM_PeriodElapsedCallback() function
 *           with implemented control by instance TIM2 - set to 1ms period
 *
 */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim) {
	if (htim->Instance == TIM2) {
		blue_led_ctl(blue_led_mode);
		green_led_ctl(green_led_mode);
		timer_2_second();
		timer_500miliseconds();
	}
}
